@@ -1,9 +1,9 @@
 /*
  * h323ep.cxx
  *
- * T38Modem simulator - main program
+ * T38FAX Pseudo Modem
  *
- * Copyright (c) 2001-2009 Vyacheslav Frolov
+ * Copyright (c) 2001-2010 Vyacheslav Frolov
  *
  * Open H323 Project
  *
@@ -24,6 +24,15 @@
  * Contributor(s): Vyacheslav Frolov
  *
  * $Log: h323ep.cxx,v $
+ * Revision 1.56  2010/10/06 16:54:19  vfrolov
+ * Redesigned engine opening/closing
+ *
+ * Revision 1.55  2010/09/29 11:52:59  vfrolov
+ * Redesigned engine attaching/detaching
+ *
+ * Revision 1.54  2010/02/27 16:33:29  vfrolov
+ * Implemented --bearer-capability option
+ *
  * Revision 1.53  2009/07/29 10:39:04  vfrolov
  * Moved h323lib specific code to h323lib directory
  *
@@ -180,7 +189,8 @@
 #include "h323ep.h"
 #include "g7231_fake.h"
 #include "t38protocol.h"
-#include "../audio.h"
+#include "audio_chan.h"
+#include "../enginebase.h"
 #include "../pmodem.h"
 #include "../drivers.h"
 
@@ -199,6 +209,7 @@ class MyH323Connection : public H323Connection
 
     // overrides from H323Connection
     AnswerCallResponse OnAnswerCall(const PString &, const H323SignalPDU &, H323SignalPDU &);
+    PBoolean OnSendSignalSetup(H323SignalPDU & setupPDU);
     void OnEstablished();
     PBoolean OnStartLogicalChannel(H323Channel & channel);
     void OnClosedLogicalChannel(const H323Channel & channel);
@@ -212,7 +223,7 @@ class MyH323Connection : public H323Connection
 
     PMutex        connMutex;
     PseudoModem * pmodem;
-    AudioEngine * audioEngine;
+    EngineBase  * userInputEngine;
 };
 ///////////////////////////////////////////////////////////////
 PString MyH323EndPoint::ArgSpec()
@@ -226,6 +237,7 @@ PString MyH323EndPoint::ArgSpec()
 
              "F-fastenable."
              "T-h245tunneldisable."
+             "-bearer-capability:"
              "G-g7231code."
              "D-disable:"            "P-prefer:"
 
@@ -272,6 +284,12 @@ PStringArray MyH323EndPoint::Descriptions()
         "  --require-gatekeeper      : Exit if gatekeeper discovery fails.\n"
         "  -F --fastenable           : Enable fast start.\n"
         "  -T --h245tunneldisable    : Disable H245 tunnelling.\n"
+        "  --bearer-capability S:C:R:P\n"
+        "                            : Bearer capability information element (Q.931)\n"
+        "                                S - coding standard (0-3)\n"
+        "                                C - information transfer capability (0-31)\n"
+        "                                R - information transfer rate (1-127)\n"
+        "                                P - user information layer 1 protocol (2-5).\n"
         "  -G --g7231enable          : Enable G.723.1 codec, rather than G.711.\n"
         "  -D --disable codec        : Disable the specified codec.\n"
         "                              Can be used multiple times.\n"
@@ -536,6 +554,34 @@ PBoolean MyH323EndPoint::Initialise(const PConfigArgs &args)
   DisableDetectInBandDTMF(TRUE);
   SetSilenceDetectionMode(H323AudioCodec::NoSilenceDetection);
 
+  if (args.HasOption("bearer-capability")) {
+    PString bc = args.GetOptionString("bearer-capability");
+    PStringArray sBC = bc.Tokenise(":", FALSE);
+    PIntArray iBC(4);
+
+    if (sBC.GetSize() == iBC.GetSize()) {
+      for (PINDEX i = 0 ; i < iBC.GetSize() ; i++)
+        iBC[i] = sBC[i].AsUnsigned();
+
+      if (iBC[0] >= 0 && iBC[0] <= 3 &&
+          iBC[1] >= 0 && iBC[1] <= 31 &&
+          iBC[2] >= 1 && iBC[2] <= 127 &&
+          iBC[3] >= 2 && iBC[3] <= 5)
+      {
+        PTRACE(3, "MyH323EndPoint::Initialise: Bearer-Capability=" << bc);
+        bearerCapability = iBC;
+      } else {
+        iBC[0] = -1;
+      }
+    } else {
+      iBC[0] = -1;
+    }
+
+    if (iBC[0] < 0) {
+      PTRACE(3, "MyH323EndPoint::Initialise: Wrong Bearer-Capability=" << bc << " (ignored)");
+    }
+  }
+
   if (args.HasOption("ports")) {
     PString p = args.GetOptionString("ports");
     PStringArray ports = p.Tokenise(",\r\n", FALSE);
@@ -645,8 +691,9 @@ PBoolean MyH323EndPoint::Initialise(const PConfigArgs &args)
 ///////////////////////////////////////////////////////////////
 
 MyH323Connection::MyH323Connection(MyH323EndPoint & _ep, unsigned callReference)
-  : H323Connection(_ep, callReference), ep(_ep),
-    pmodem(NULL), audioEngine(NULL)
+  : H323Connection(_ep, callReference), ep(_ep)
+  , pmodem(NULL)
+  , userInputEngine(NULL)
 {
 }
 
@@ -655,14 +702,6 @@ MyH323Connection::~MyH323Connection()
   cout << "Closing connection" << endl;
 
   if (pmodem != NULL) {
-    if (t38handler != NULL) {
-      PAssert(PIsDescendant(t38handler, T38Protocol), PInvalidCast);
-      pmodem->Detach((T38Protocol *)t38handler);
-    }
-
-    if (audioEngine != NULL)
-      pmodem->Detach(audioEngine);
-
     PStringToString request;
     request.SetAt("command", "clearcall");
     request.SetAt("calltoken", GetCallToken());
@@ -673,8 +712,8 @@ MyH323Connection::~MyH323Connection()
     ep.PMFree(pmodem);
   }
 
-  if (audioEngine != NULL)
-    delete audioEngine;
+  if (userInputEngine != NULL)
+    ReferenceObject::DelPointer(userInputEngine);
 }
 
 void MyH323Connection::OnEstablished()
@@ -697,11 +736,6 @@ PBoolean MyH323Connection::Attach(PseudoModem *_pmodem)
 
   pmodem = _pmodem;
 
-  if (audioEngine == NULL)
-    audioEngine = new AudioEngine(pmodem->ptyName());
-
-  pmodem->Attach(audioEngine);
-
   return TRUE;
 }
 
@@ -716,11 +750,19 @@ OpalT38Protocol * MyH323Connection::CreateT38ProtocolHandler()
    * we can't have more then one t38handler per connection
    * at the same time and we should delete it on connection clean
    */
-  if( t38handler == NULL ) {
+  if (t38handler == NULL) {
     PTRACE(2, "MyH323Connection::CreateT38ProtocolHandler create new one");
-    t38handler = new T38Protocol(pmodem->ptyName());
-    ep.SetOptions(*this, t38handler);
-    pmodem->Attach((T38Protocol *)t38handler);
+
+    T38Protocol *t38protocol = new T38Protocol(pmodem);
+
+    if (t38protocol && t38protocol->IsOK()) {
+      ep.SetOptions(*this, t38protocol);
+
+      t38handler = t38protocol;
+    } else {
+      delete t38protocol;
+      PTRACE(1, "MyH323Connection::CreateT38ProtocolHandler can't create");
+    }
   }
   return t38handler;
 }
@@ -796,6 +838,23 @@ H323Connection::AnswerCallResponse
   return AnswerCallDenied;
 }
 
+PBoolean MyH323Connection::OnSendSignalSetup(H323SignalPDU & setupPDU)
+{
+  const PIntArray &bearerCapability = ep.BearerCapability();
+
+  if (!bearerCapability.IsEmpty()) {
+    PTRACE(3, "MyH323Connection::OnSendSignalSetup: Set Bearer capability '" << bearerCapability << "'");
+
+    setupPDU.GetQ931().SetBearerCapabilities(
+        Q931::InformationTransferCapability(bearerCapability[1]),
+        bearerCapability[2],
+        bearerCapability[0],
+        bearerCapability[3]);
+  }
+
+  return H323Connection::OnSendSignalSetup(setupPDU);
+}
+
 PBoolean MyH323Connection::OnStartLogicalChannel(H323Channel & channel)
 {
   myPTRACE(1, "MyH323Connection::OnStartLogicalChannel ch=" << channel << " cp=" << channel.GetCapability() << " sid=" << channel.GetSessionID() << " " << channel.GetDirection());
@@ -821,23 +880,38 @@ void MyH323Connection::OnClosedLogicalChannel(const H323Channel & channel)
   myPTRACE(1, "MyH323Connection::OnClosedLogicalChannel ch=" << channel << " cp=" << channel.GetCapability() << " sid=" << channel.GetSessionID() << " " << channel.GetDirection());
 }
 
-PBoolean MyH323Connection::OpenAudioChannel(PBoolean isEncoding, unsigned /* bufferSize */, H323AudioCodec & codec)
+PBoolean MyH323Connection::OpenAudioChannel(PBoolean isEncoding, unsigned /*bufferSize*/, H323AudioCodec & codec)
 {
-  PTRACE(2, "MyH323Connection::OpenAudioChannel " << codec);
+  PTRACE(2, "MyH323Connection::OpenAudioChannel " << codec << (isEncoding ? " encoding" : " decoding"));
 
-  PAssert(audioEngine != NULL, "audioEngine is NULL");
+  if (pmodem == NULL)
+    return FALSE;
 
-  codec.AttachChannel(audioEngine, FALSE);
+  ModemAudioChannel *audioChannel = new ModemAudioChannel(isEncoding, pmodem);
+
+  if (!audioChannel || !audioChannel->IsOK()) {
+    delete audioChannel;
+    PTRACE(1, "MyH323Connection::OpenAudioChannel can't create ModemAudioChannel");
+  }
+
+  codec.AttachChannel(audioChannel, TRUE);
 
   return TRUE;
 }
 
 void MyH323Connection::OnUserInputString(const PString & value)
 {
-  PAssert(audioEngine != NULL, "audioEngine is NULL");
+  if (userInputEngine == NULL) {
+    if (pmodem == NULL)
+      return;
 
-  audioEngine->WriteUserInput(value);
+    userInputEngine = pmodem->NewPtrUserInputEngine();
+
+    if (userInputEngine == NULL)
+      return;
+  }
+
+  userInputEngine->WriteUserInput(value);
 }
-
-// End of File ///////////////////////////////////////////////////////////////
+///////////////////////////////////////////////////////////////
 
